% File: inputdata.m 
% Author: Dennis Thomas 
% Date created: October 8, 2013
% Last Modified:  July 29, 2017

% This file accepts inputs for the isd3 simulations through the function 
%  "inputdata", such as inputs for
% 1) liquid media characteristics
% 2) particle characteristics,
% 3) simulation grid and boundary specifications,
% 4) time integration parameters,
% 5) dissolution model parameters, and
% 6) cellular ion uptake model parameters

%-----------------------HOW TO RUN ISD3------------------------------------

% 1. The user enters values for each parameter, all of which
% can be identified by the naming convention: parameter.name

% 2. The user sets the option for parameter.agg_yn to consider whether particles of size
% greater than the primary particle size are to be treated as agglomerates
% or not.

% 3. The user (recommended) saves this file with a unique
% name associated with the experiment, for example:
% 20nmsilver12pt5conc_07302017_1.m

% 4. Right clicking and running the isd3.m file will initiate the
% simulations.
% ------------------------------------------------------------------------
 
% Function to specify the input parameters for the ISD3 simulation
function [parameter,diss_parameters,uptake_parms] = inputdata

% ------------------- PARAMETER INPUT BEGINS HERE--------------------------
%If you are using a mac to run isd3, the  xlswrite function will not work. 
%Therefore, use the xlwrite function provided in a separate file by 
% setting the value of use_xlswrite to "yes". allowed inputs: yes, no.
parameter.use_xlwrite = 'no';  

% Number of processors to use for parallel for loop runs (using parfor).
% Minimum value is 1; maximum value should not exceed the number of grid 
% points along the particle diameter axis
parameter.num_procs =10; 

% Name of the .mat file to use for saving the output variables of 
% the simulation. 
parameter.results_matfile = 'isd3v12_0pt7_110nm_02212016_1';

% LIQUID MEDIA CHARACTERISTICS --------------------------------------------
parameter.media_height      = 0.00315;   % Dish Depth (m)
parameter.media_volume      = 3;        % Volume of the media, mL
parameter.media_temperature = 310.0;    % Temperature (K)
parameter.media_viscosity   = 0.00074;  % Water viscosity (N s/m2)
parameter.media_density  	= 1.0;      % Density of media (g/mL)


parameter.media_surfacearea = parameter.media_volume*(1e-6)/...
                              parameter.media_height; % Surface area of the
                             % medium perpendicular to x axis direction (m^2)

% PARTICLE CHARACTERISTICS ------------------------------------------------
% Primary Particle Characteristics ----------------------------------------
parameter.ptcl_diameter     = 110;     % Diameter of the undissolved primary
                                        %  particle(nm)
parameter.ptcl_density      = 10.0;    % Density of the undissolved primary
                                        % particle (g/mL) 
parameter.conc  = 0.7;   % Initial concentration of particles in the liquid 
                         %media (ug/mL)
%Protein-Coated Particle Characteristics-----------------------------------
parameter.ptcl_protein_density = 1.914; % density of protein-coated particles (g/mL)
parameter.ptcl_protein_thickness = 22.5;% thickness of the protein layer 
                                        % coating the particle surface (nm)

% Initial Particle Size Distribution---------------------------------------
parameter.size_distribution_type = 'normal';  % Type of size distribution
                                                % allowed inputs:  log-normal, normal
parameter.mu_normal = 155.0; % Average diameter of normal distribution (nm)
parameter.sigma_normal=0.1; % Standard deviation of normal distribution (nm)
parameter.gsd_lognormal = 1.66; % Geometric standard deviation of log-normal distribution 
parameter.gmean_lognormal = 53.142; % Geometric mean of log-normal distribution (equivalent to median)
parameter.mu_lognormal = log(parameter.gmean_lognormal);
parameter.sigma_lognormal = log(parameter.gsd_lognormal);
switch parameter.size_distribution_type
    case 'normal'
        parameter.ptcl_avg_diameter =  parameter.mu_normal - 2*parameter.ptcl_protein_thickness;
parameter.ptcl_sd_diameter = parameter.sigma_normal;
        parameter.ptcl_max_diameter = parameter.ptcl_avg_diameter +3*parameter.ptcl_sd_diameter;     % Maximum value of the particle diameter (nm); e.g, 10 nm above the average value
    case 'log-normal'
        parameter.ptcl_avg_diameter =  parameter.gmean_lognormal - 2*parameter.ptcl_protein_thickness; 
        parameter.ptcl_sd_diameter = parameter.gsd_lognormal ;
parameter.ptcl_max_diameter = exp(parameter.mu_lognormal+parameter.sigma_lognormal) - 2*parameter.ptcl_protein_thickness;
end
% The mean and sd values of the diameter will get modified if the size
% distirbution is log-normal distribution.

parameter.ptcl_min_diameter = 110;    % Minimum value of the particle 
                                       %diameter (nm), initially in the system.
% Particle Agglomeration Characteristics ----------------------------------
parameter.DF = 2.3;         % Fractal Dimension (only relevant to agglomerates)
parameter.PF = .637;        % Packing Factor (only relevant to agglomerates)


parameter.agg_yn = 'agg_n';     % Option to set whether to consider particles
                                % as agglomerates or not  
                                                    
% agg_yn options:
%    agg_n:     
%   Treat secondary particles with size greater than the primary undissolved
%   particle size as non-porous particles (i.e., they are not agglomerates).
%   Particles with size less than primary particle size are always
%   treated as non-porous particles.
%
%   'agg_y':    
%   Treat secondary particles with size greater than primary undissolved
%   particle size as agglomerates.Particles with size less than primary 
%   particle size are always treated as non-porous particles.

% GRID SPECIFICATIONS AND BOUNDARY CONDITIONS -----------------------------
parameter.del_dp= 1.0;          % grid spacing along diameter axis (nm)
parameter.dp_l = 10;            % lowest particle diameter along diameter axis (nm)
parameter.dp_u = parameter.ptcl_max_diameter; % highest particle diameter (based on the size distribution)
% The value gets modified (in isd3_semi_implicit.m) if the input size 
% distribution is a log-normal distribution.

parameter.Nx = 1000;   % Number of grid points in the x-axis direction

% Type of boundary condition at the cell surface (bottom, x=L); the 'right'
% boundary condition to be set in the pdepe solver.
 parameter.bctype = 'sticky';    % allowed values: 'sticky', 'noflux'
 
% TIME INTEGRATION INPUTS -------------------------------------------------
parameter.tmaxh = 24; % Total simulation time (hours)
parameter.Nt = 1000; % Number of time steps (e.g., 1000)
parameter.tol = 1; % Tolerance value used to satisfy the convergence 
                   % criterion for surface area at each time step


% DISSOLUTION MODEL INPUTS-------------------------------------------------

diss_parameters.P0 = 10; % percentage of FBS in solution (%)
diss_parameters.kf =0.0006e-14; % rate constant for dissolution of 
                                % particles to free ions (in mL/(nm^2 hr)) 
                                %[diffusivity,D/distance,h]
diss_parameters.Cfsat = 1 ; % saturated concentration of free ions in the 
                            %media (in ug/ml)
diss_parameters.kp = 0.003e-14; % rate constant for slow transfer of ions 
                                % from the particle surface to the proteins
                                %(in mL/(nm^2 hr))
diss_parameters.kp2 = 0.1e-14;  % rate constant for the initial fast 
                                % transfer of ions from the particle surface
                                % to the proteins (in mL/(nm^2 hr))
diss_parameters.kfp =0.0114; % rate constant for the transfer of free ions 
                             % from solution to the proteins (in ug^-1 ml h^-1)
diss_parameters.kp2f =0.016; % rate constant for the transfer of ions from
                             % the protein-bound state to the free ion state
                             %(ug^-1 mL h^-1)
diss_parameters.n = 0.4;    % concentration of binding sites available on 
                            % the proteins per FBS for the slow binding 
                            % period of the ions% (in ug mL^-1 FBS%^-1)
diss_parameters.n2 = 1.2534*diss_parameters.n*power(diss_parameters.P0,-0.403); 
                            % concentration of binding sites available on 
                            % the proteins per FBS for the fast binding 
                            % period of the ions% (in ug mL^-1 FBS%^-1)

%-----------------CELLULAR ION UPTAKE RATE INPUTS -------------------------
% 2 - cells, 1 - media
uptake_parms.sa2 = parameter.media_surfacearea*1e4; % cm^2          
uptake_parms.dis2 = 7.8e-7; % thickness of cell membrane (cm)
uptake_parms.d12 = 9.02e-9; % diffusion coefficient (cm^2/h)
uptake_parms.pc21 = 25.8; % cell:media silver ion partition coefficient (dimensionless)
uptake_parms.cellv = 1100*(1.76*1e6)*1e-12; % cell volume 

% -----------------------PARAMETER INPUT ENDS HERE-------------------------
end
