% ISD3 main program
% Author: Dennis G. Thomas
% Created date: December 2013
% Last modified: July 29, 2017

% Main function of isd3 software program
% Right clicking and running this m file will initiate the simulations.

function isd3

time_start = tic; 

% Get inputs for
% 1) liquid media characteristics
% 2) particle characteristics,
% 3) simulation grid and boundary specifications,
% 4) time integration parameters,
% 5) dissolution model parameters, and
% 6) cellular ion uptake model parameters

[parameter,diss_parameters,uptake_parms] = inputdata;

% Set boundary conditions
switch parameter.bctype
    case 'sticky'  % Concentration = 0 at the bottom wall
        bctype = 1;
    case 'noflux'  % flux = 0 at the bottom wall
        bctype = 0;
end
% Add path to poi_library files if use_xlswrite is set to 'yes'
if(strcmp(parameter.use_xlwrite,'yes')==1)
    javaaddpath('poi_library/poi-3.8-20120326.jar');
    javaaddpath('poi_library/poi-ooxml-3.8-20120326.jar');
    javaaddpath('poi_library/poi-ooxml-schemas-3.8-20120326.jar');
    javaaddpath('poi_library/xmlbeans-2.3.0.jar');
    javaaddpath('poi_library/dom4j-1.6.1.jar');
end
% Get the name of the .mat file to use for saving the output variables of 
% the simulation. 
results_matfile = parameter.results_matfile;

disp('Writing out the parameters ');
fname = results_matfile;
writeParameters(parameter,diss_parameters,uptake_parms,fname);

% ------SET UP THE INITIAL NUMBER DENSITY FRACTION OF PARTICLES -------
% Variables and definitions
%   - x         :   Location of the particles perpendicularly above the
%                   cell surface (nm)
%   - Dp        :   Particle diameter (nm)
%   - t         :   Time(hr)
%   - dp_avg    :   Average particle diameter (nm)
%   - dp_sd     :   Standard deviation of particle size distribution
%   - n(Dp;x,0) :   Particle number fraction density (/nm^2 units) at time,t = 0,
%                   at any point (x,Dp) in the x-Dp parameter space.
%
%       n(Dp;x,0) is calculated as the number of particles per unit
%       area in x-Dp space,divided by the (total) initial
%       number of particles in the liquid medium
%       (calculated based on the initial particle concentration).
%   - n(Dp;x,t) :   Particle number fraction density (/nm^2 units) at time,t,
%                   at any point (x,Dp) in the x-Dp parameter space.
%   - N(Dp;x,t) :   Particle number density (/nm^2 units) at time,t,
%                   at any point (x,Dp) in the x-Dp parameter space.
%   - delx      :   Uniform grid spacing (nm) used for discretizing the x dimension.
%   - f(Dp)     :   Gaussian function for the initial number fraction density at
%                   time t = 0 along Dp, i.e.,
%
%       f(Dp) = exp(-(Dp-dp_avg)^2/(2*dp_sd^2))/(dp_sd*sqrt(2pi))
%
% Therefore,
%
%       n(Dp;x,0) = f(Dp)*fac_1, where the value of fac_1 is assigned such that:
%
%   1. the initial number fraction density is uniformly distributed along
%   the 'x' dimension; and, that
%   2. the integral of n(Dp;x,0) in x-Dp space is equal to 1.
% -------------------------------------------------------------------------

C0 = parameter.conc;        % Initial mass concentration of particles (ug/mL)
V = parameter.media_volume; % Volume of the liquid medium(ml)
L = parameter.media_height; % Height of the liquid medium(m)
SA = parameter.media_surfacearea; %Surface area of the medium perpendicular to
% x axis direction (m^2)

del_dp = parameter.del_dp; % grid spacing along dp coordinate axis (nm)
size_distribution_type = parameter.size_distribution_type;
switch size_distribution_type
    case 'normal'
        mu = parameter.ptcl_avg_diameter ; % average diameter of the particles at time t = 0 (nm)
        sigma= parameter.ptcl_sd_diameter;  % standard deviation of the particle diameter
        
    case 'log-normal'
        mu = log(parameter.ptcl_avg_diameter );
        sigma=log(parameter.ptcl_sd_diameter);
        
end

dp_u = parameter.dp_u ; % upper limit of particle diameter (nm).
dp_l = parameter.dp_l   ; % lower limit of particle diameter (nm).

dp_avg = parameter.ptcl_avg_diameter ; % average diameter of the particles at time t = 0 (nm) - geometric mean
dp_sd = parameter.ptcl_sd_diameter;

dp_prim = parameter.ptcl_diameter;      % Primary particle diameter (nm)

Nx = parameter.Nx;                   % Number of grid points in the x-axis direction
mesh_output = setMesh(dp_u,dp_l,dp_avg,del_dp,Nx,L);

Ndp = mesh_output.Ndp;
Nx = mesh_output.Nx;
dp =mesh_output.dp;
delx_dim=mesh_output.delx_dim;
x=mesh_output.x;
x_nm=mesh_output.x_nm;
% x_dim = mesh_output.x_dim;
% dp_dim=mesh_output.dp_dim;
% del_dp_dim=mesh_output.del_dp_dim;
delx_m = x(2)-x(1);

clear mesh_output;

% log file
% logfile = [parameter.results_matfile,'.log'];
% logfid = fopen(logfile, 'w');

%----------CALCULATE DIAMETER-DEPENDENT PROPERTIES OF PARTICLES------------
% Evaluate the parameters (dependent on Dp) over very grid point in Dp space
% - diffusivity (m^2/s)
% - sedimentation velocity (m/s)
% - particle density (g/cc)

% Get the input parameters needed for the calculation

temperature = parameter.media_temperature ; % Temperature (K)
viscosity  = parameter.media_viscosity; % viscosity of the medium (N s/m2)
PF = parameter.PF;  % Packing Factor (only relevant to agglomerates)
DF = parameter.DF; % Fractal Dimension (only relevant to agglomerates)
dw = parameter.media_density; % Density of media (g/mL)
rhop_prim = parameter.ptcl_density;     % Primary particle density (g/cc)


% ------CALCULATE THE INITIAL NUMBER FRACTION DENSITY IN DP SPACE----------

dp_init_l = parameter.ptcl_min_diameter;
dp_init_u = dp_u;

outputNumFracDensDp = initializeNumFracDensDpspace(size_distribution_type,Ndp,dp,mu,sigma,dp_init_l,dp_init_u);
n0_dp=outputNumFracDensDp.n0_dp;
% area_n0_dp = outputNumFracDensDp.area_n0_dp;
% area_n0_dp_norm=outputNumFracDensDp.area_n0_dp_norm;
% avgdp = outputNumFracDensDp.avg_dp;
clear outputNumFracDensDp;

% Option to decide whether or not to treat particles of diameter greater than
% the primary particle diameter as agglomerates.
agg_yn = parameter.agg_yn;

diff_dp = zeros(Ndp,1);     % Diffusivity (m^2/s) as a function of particle diameter
vsed_dp = zeros(Ndp,1);     % Sedimentation velocity (m/s) as a function of particle diameter
rho_dp = zeros(Ndp,1);
pth = parameter.ptcl_protein_thickness;
rhop_protein = parameter.ptcl_protein_density;
for i=1:Ndp
    diff_dp(i,1) = diffusivity(temperature,viscosity,dp(i,1),pth);
    [vsed_dp(i,1),rho_dp(i,1)] = sedimentation_velocity(agg_yn,DF,PF,dw,viscosity,dp(i,1),dp_prim,rhop_prim,pth,rhop_protein);
end

% for i=1:Ndp
%     diff_dp(i,1) = diffusivity(temperature,viscosity,avgdp(i,1));
%     [vsed_dp(i,1),rho_dp(i,1)] = sedimentation_velocity(agg_yn,DF,PF,dw,viscosity,avgdp(i,1),dp_prim,rhop_prim);
% end
%-----CALCULATE THE INITIAL NUMBER OF PARTICLES IN THE LIQUID MEDIUM-------

% Calculate initial number of particles,N0, from the initial particle
% mass concentration, C0, and the initial number fraction density, n0_dp.
% - Define variable, vpi - volume of particle with diameter dp_i
% - C0 is in ug/ml, V is in ml, rho_dp is in g/ml, vpi is in nm^3
% - 1 ml = 1e21 nm^3
% - 1 ug = 1e-6 g


temp = zeros(Ndp,1);%n0_dp;       % Temporary variable to hold the initial number
% fraction density values in Dp space

for i = 1:Ndp
    
    vpi = pi*dp(i,1)*dp(i,1)*dp(i,1)/6; % Volume of particle with diameter dp_i, in nm^3
    rhopi = rho_dp(i,1)*(1e-21);        % Density of particles with diameter dp_i, in g/nm^3
    temp(i,1) = n0_dp(i,1)*vpi*rhopi; % Mass of particles with diameter dp_i, in g
end
denom_N0 = integrate_density_dp(temp,dp);   % Area under the temp vs Dp curve

N0 = C0*V*(1e-6)/denom_N0;  % Initial number of particles
clear temp;

%-------CALCULATE THE INITIAL NUMBER DENSITY IN THE LIQUID MEDIUM----------

% [num_dens_dp0,num_dens_init,area_num_dens] = ...
%     initializeNumDensXDPSpace(N0,dp,x_nm,n0_dp,Nx,Ndp,delx_dim,L);
[~,num_dens_init,~] = ...
    initializeNumDensXDPSpace(N0,dp,x_nm,n0_dp,Nx,Ndp,delx_dim,L);

% Store the parameters needed for defining the initial particle number fraction density
% function in x-Dp space.
i_dpavg = round((dp_avg-dp_l)/del_dp + 1); %ceil(1/del_dp_dim);
%i_dpavg = find(dp==dp_avg);
num_dens_init.idpavg = i_dpavg;
num_dens_init.dpu = dp(end,1);
num_dens_init.dpl = dp(1,1);
num_dens_init.dpavg = dp_avg;
num_dens_init.dpsd = dp_sd;
num_dens_init.size_distribution_type = parameter.size_distribution_type;

%---------------CALCULATE THE TIME SCALES OF THE PROBLEM-------------------

% diff_max = max(diff_dp) ; % Maximum value of diffusivity, m^2/s
% vsed_max = max(vsed_dp); % Maximum value of the sedimentation velocity, m/s

% Diffusion time scale (s)

%time.diff  = L*L./diff_dp;  % Diffusion time (s) as a function of particle diameter
time.diff  = delx_m*delx_m./diff_dp;  % Diffusion time (s) as a function of particle diameter
time.diff_max = max(time.diff); % highest diffusion time (s)
time.diff_min = min(time.diff); % lowest diffusion time (s)
time.diff_scale = time.diff(i_dpavg,1);  % Diffusion time scale (s)
%time.delx_diff_scale = time.diff_scale*delx_dim*delx_dim/3600; % diffusion time within one grid spacing (h)

% Sedimentation time scale (s)
%time.sed = L./vsed_dp;  % Sedimentation velocity (seconds) as a function of particle diameter
time.sed = delx_m./vsed_dp;  % Sedimentation velocity (seconds) as a function of particle diameter
time.sed_max = max(time.sed);   % Maximum sedimentation time (s)
time.sed_min = min(time.sed);   % Maximum sedimentation time (s)
time.sed_scale = time.sed(i_dpavg,1); % Sedimentation time scale (s)

% Dissolution time scale (s)
% - Note: rho_dp is in g/cc units, dp is in nm units.

% diss_rate = parameter.solubility_rate ; % Rate constant for dissolution (in ug/nm^2/hr)
% time.diss = (rho_dp.*dp)/(diss_rate*10000/36); % Dissolution time (seconds) as a function of particle density
% time.diss_max = max(time.diss);     % Maximum value of dissolution time (s)
% time.diss_scale = time.diss(i_dpavg,1); % Dissolution time scale (s)



%---------------------TIME INTEGRATION STARTS HERE-------------------------

%----- Get the time integration parameters -----
tmaxh = parameter.tmaxh; % Maximum simulation time (hours)
tmaxs = tmaxh*60*60;    % Maximum simulation time (seconds)

%time_scale = min([time.diff_scale time.sed_scale time.diss_scale]);

% Non-dimensionalize the time variables using the sedimentation time scale
% - Time parameters are stored in struct variable, time. The dimensionless
% time is not used for solving the equations.
time.scale_sec = time.sed_scale;
tmaxs_dim = tmaxs/time.scale_sec;

delt_dim = tmaxs_dim/parameter.Nt;      % time integration step (dimensionless)
delt_h = delt_dim*time.scale_sec/3600;   % time integration step (hours)

time.delt_h = delt_h;
% dispout = ['time integration step size = ',num2str(time.delt_h),' hours.'];
% fprintf(logfid,'%s\n',dispout);
disp(['time integration step size = ',num2str(time.delt_h),' hours.']);

% d1 = time.delt_h - time.diff_min/3600;
% d2 = time.delt_h - time.sed_min/3600;
% if(d1 > 0 || d2 > 0)
%     dispout = ['time integration step size may need to be reduced below ' num2str(max(d1,d2)) 'hours.'];
% fprintf(logfid,'%s\n',dispout);
%    % disp(['time integration step size may need to be reduced below ' num2str(max(d1,d2)) 'hours.']);
% end

% Number of time points
Nt = parameter.Nt+1;
time.values_hr = zeros(Nt,1);

%fid = fopen('runoutput.txt','w+');
%fprintf(fid,'time step size (hours) = %d\n',delt_h);
%fclose(fid);

% ----- Dissolved ion concentration -------
Cdiss = zeros(Nt,1);    % total dissolved ion concentration in the liquid media, ug/ml

Cdiss_f = zeros(Nt,1); % dissolved free ion concentration in the liquid media, ug/ml
Cdiss_p = zeros(Nt,1); % dissolved protein-bound ion concentration in the liquid media, ug/ml
Cdiss_cell = zeros(Nt,1); % dissolved ion concentration in the cells, ug/ml

% Initial dissolved concentration is set to zero. Later versions may
% include specifying these values in the parameter file (inputdata.m).
Cdiss_f(1,1) = 0;
Cdiss_p(1,1) = 0;
Cdiss_cell(1,1) = 0;
Cdiss(1,1) = Cdiss_f(1,1) + Cdiss_p(1,1); % total initial concentration for dissolved solute in the liquid medium
Ctotal = zeros(Nt,1);
Ctotal(1,1) = C0;
%------Particle number density function --------
% Set up the variables to hold the particle number density function at every time
% step in x-Dp space, and its first derivative with respect to Dp
num_dens_txdp(Nt,1) = struct;
derdp_num_dens_txdp(Nt,1)  = struct;

for i=1:Nt
    num_dens_txdp(i,1).values = zeros(Nx,Ndp); % n as a function of time, x, and Dp
    derdp_num_dens_txdp(i,1).values = zeros(Nx,Ndp); % d(n*dDp/dt)/dDp as a function time, x, and Dp
end

disp('The variable solved by time integration is the number density function.');
disp('The units of number density function is #/nm^2');

% total_num_particles = N0;
num_dens_txdp(1).values(:,:) = num_dens_init.data;


%---- Surface area of particles ----

% Total surface area of particles divided by total number of
% particles (nm^2)
areat_N0_ratio = zeros(Nt,1);

% Total surface area of particles (nm^2)
areat = zeros(Nt,1);

% Calculate areat_N0_ratio and areat
[areat_N0_ratio(1,1),areat(1,1)] = surface_area(agg_yn,rho_dp,Nx,Ndp,dp,num_dens_txdp(1).values,x_nm,N0,rhop_prim,dp_prim);

% ---- Begin time integration -----
tol = parameter.tol;                 % tolerance value to use for satisfying convergence criterion at each time step
residual = zeros(Nx*Ndp,1);     % residual values calculated between consecutive iterations in a given time step


% Cdissq = 0;
% Cdisscellq = 0;
% Cdissqminus1 = 0;
% Cdisscellqminus1=0;
% partNumq = total_num_particles;
% partNumqminus1 = 0;
partSAq = 0;
partSAqminus1 =  areat(1,1);

% set the array for recording the accepted error in convergence at every
% time step.

errort=zeros(Nt,1);

num_dens_dp_cell(Nt,1) = struct;
for i=1:Nt
    num_dens_dp_cell(i,1).values = zeros(Ndp,1);
end



num_procs = parameter.num_procs;
parpool(num_procs);
for i=2:Nt
    q = 0;          % index for
    error = 1000;       % set the error to a large value at the beginning of each time step.
    %     error_conc = 1000;  % Set the error to a large value for the absolute
    % difference between total concentration and
    % initial concentration.
    time.values_hr(i,1)=(i-1)*delt_h;
    time.values_hr(i,1);
    
    disp(['Time = ', num2str(time.values_hr(i,1)),' hours']);
    %      dispout = ['Time = ', num2str(time.values_hr(i,1)),' hours'];
    % fprintf(logfid,'%s\n',dispout);
    %fid = fopen('runoutput.txt','a+');
    %fprintf(fid,'Time = %d hours\n',time.values_hr(i,1));
    % 	fclose(fid);
    
    % Differentiate N_i-1 with respect to Dp
    % - derdp_num_frac_dens_txdp : dN/dDp
    
    %     derdp_num_dens_txdp(i-1).values(:,:) = derdp(num_dens_txdp(i-1).values(:,:),del_dp,Ndp);
    
    
    %coeff(i-1) = 2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(i-1,1),Cdiss_p(i-1,1));
    % unit of coeff is nm hr^-1 ug ml^-1
    error_conc_tol = 1e-5;
    while (error > tol || q < 2)        % convergence criterion used to stop the iterations
        error_conc= 1000;
        
        error_conc_prev = error_conc;
        while(error_conc>error_conc_tol)
            q = q + 1;
            % V ml, delt_h hr
            
            if(q>1)
                num_dens_txdp_q1(:,:) =  num_dens_txdp(i).values(:,:);
                %                 Cdissqminus1 = Cdissq; %Cdiss(i,1);
                %                 Cdisscellqminus1 =Cdisscellq;% Cdiss_cell(i,1);
                %                 partNumqminus1 = partNumq; %integrate_density_xdp(num_dens_txdp(i).values,dp,x_nm);
                % [areatbyN0,partSAqminus1] = surface_area(Nx,Ndp,dp,num_dens_txdp(i).values(:,:),x_nm,N0,dp_prim);
                partSAqminus1 = partSAq;
            end
            
            % Adam-Bashford explicit method to solve for N_{i+1/2;q}
            % Equation:  dN_{i+1/2;q}/dt =
            % 2*k/(rho_p)*(1-Cdiss_{i+1;q}/Ksp)*dN_[i+1/2;q}/dDp
            
            % Units
            % Cdiss - dissolved concentration, ug/ml
            % rho_dp - particle density, g/ml
            % Ksp - solubility constant, ug/ml
            % diss_rate - Rate constant of dissolution (in ug/nm^2/hr)
            % kc - cellular uptake rate of ions, 1/hr (1st order)
            
            if(q==1)
                
                [areat_N0_ratio(i,1),areat(i,1)] = surface_area(agg_yn,rho_dp,Nx,Ndp,dp,...
                    num_dens_txdp(i-1).values(:,:),x_nm,N0,rhop_prim,dp_prim);
                
                [Cdiss(i,1),Cdiss_f(i,1),Cdiss_p(i,1),Cdiss_cell(i,1)] = ...
                    solve_Cdiss_model1(diss_parameters,uptake_parms,V,...
                    areat(i,1),delt_h,Cdiss_f(i-1,1),Cdiss_p(i-1,1),...
                    Cdiss_cell(i-1,1));
                coeffd = 2*DdpDt_dissterm_model1(diss_parameters,...
                    Cdiss_f(i,1),Cdiss_p(i,1),V);
                % Note: the -ve sign inside the DdpDt_diss_term_model1 is not
                % included, since it will multiply the -ve sign outside the
                % 2*d(n*DdpDt_dissterm_model1)/dDp.
                
                derdp_values = zeros(Nx,Ndp);
                dens_values = num_dens_txdp(i-1).values;
                
                switch agg_yn
                    case 'agg_n'
                        for j=1:Ndp
                            %                     for k=1:Nx
                            
                            % derdp_num_dens_txdp(i).values(k,j) = num_dens_txdp(i-1).values(k,j)*coeffd/(rho_dp(j,1)*10^(-15));
                            derdp_values(:,j) = dens_values(:,j)*coeffd/(rho_dp(j,1)*10^(-15));
                            
                            
                            %                     end
                        end
                    case 'agg_y'
                        for j=1:Ndp
                            
                            %                     for k=1:Nx
                            if(dp(j,1)>dp_prim)
                                derdp_values(:,j) = dens_values(:,j)*coeffd*(dp(j,1)/dp_prim)/(rhop_prim*10^(-15));
                            else
                                derdp_values(:,j) = dens_values(:,j)*coeffd/(rho_dp(j,1)*10^(-15));
                            end
                            
                            %                     end
                        end
                        
                end
                % set the derivative to zero at the lowest value, to not allow
                % particles of the lowest size to dissolve.
                derdp_values(:,1) = 0;
                derdp_num_dens_txdp(i).values = derdp_values;
                
                % Take the derivative with respect to Dp.
                derdp_num_dens_txdp(i).values(:,:) = derdp(derdp_num_dens_txdp(i).values(:,:),del_dp,Ndp);
                % set the derivative to zero at the lowest value, to not allow
                % particles of the lowest size to dissolve.
                % derdp_num_dens_txdp(i).values(:,1) = 0;
                if(i==2)
                    
                    coeffd = 2*DdpDt_dissterm_model1(diss_parameters,...
                        Cdiss_f(i-1,1),Cdiss_p(i-1,1),V);
                    % Note: the -ve sign inside the DdpDt_diss_term_model1 is not
                    % included, since it will multiply the -ve sign outside the
                    % 2*d(n*DdpDt_dissterm_model1)/dDp.
                    
                    derdp_values = zeros(Nx,Ndp);
                    dens_values = num_dens_txdp(i-1).values;
                    
                    switch agg_yn
                        case 'agg_n'
                            for j=1:Ndp
                                %                     for k=1:Nx
                                
                                % derdp_num_dens_txdp(i).values(k,j) = num_dens_txdp(i-1).values(k,j)*coeffd/(rho_dp(j,1)*10^(-15));
                                derdp_values(:,j) = dens_values(:,j)*coeffd/(rho_dp(j,1)*10^(-15));
                                
                                
                                %                     end
                            end
                        case 'agg_y'
                            for j=1:Ndp
                                
                                %                     for k=1:Nx
                                if(dp(j,1)>dp_prim)
                                    derdp_values(:,j) = dens_values(:,j)*coeffd*(dp(j,1)/dp_prim)/(rhop_prim*10^(-15));
                                else
                                    derdp_values(:,j) = dens_values(:,j)*coeffd/(rho_dp(j,1)*10^(-15));
                                end
                                
                                %                     end
                            end
                            
                    end
                    % set the derivative to zero at the lowest value, to not allow
                    % particles of the lowest size to dissolve.
                    derdp_values(:,1) = 0;
                    derdp_num_dens_txdp(i-1).values = derdp_values;
                    
                    % Take the derivative with respect to Dp.
                    derdp_num_dens_txdp(i-1).values(:,:) = derdp(derdp_num_dens_txdp(i-1).values(:,:),del_dp,Ndp);
                    
                    % Take the derivative with respect to Dp.
                    % derdp_num_dens_txdp(i-1).values(:,:) = derdp(derdp_num_dens_txdp(i-1).values(:,:),del_dp,Ndp);
                    % du/dt = f(t,u)
                    %Explicit 1st order Adam-Bashforth (forward Euler)
                    % u(n+1) = u(n) + delta_t*f(t(n),u(n))
                    
                    num_dens_txdp(i).values(:,:) = ...
                        num_dens_txdp(i-1).values(:,:)+ ...
                        delt_h*derdp_num_dens_txdp(i-1).values(:,:);
                    
                else
                    
                    %                         %  Explicit 2nd order Adam-Bashforth for
                    %                         % an equation of the form, du/dt = f(t,u)
                    %                         % u(n+1) = u(n) +
                    %                         % delta_t*[3*f(t(n),u(n))-f(t(n-1),u(n-1))]/2
                    
                    num_dens_txdp(i).values(:,:) = num_dens_txdp(i-1).values(:,:)+ delt_h*(3*derdp_num_dens_txdp(i-1).values(:,:)-derdp_num_dens_txdp(i-2).values(:,:))/2;
                    
                end
                
                
            elseif (q>=2)
                % Differentiate N_{i+1;q} with respect to Dp at the qth iteration,
                
                
                % [areat_N0_ratio(i,1),areat(i,1)] = surface_area(Nx,Ndp,dp,num_dens_txdp(i).values(:,:),x_nm,N0,dp_prim);
                % The area has already been updated.
                
                [Cdiss(i,1),Cdiss_f(i,1),Cdiss_p(i,1),Cdiss_cell(i,1)] = solve_Cdiss_model1(diss_parameters,uptake_parms,V,areat(i,1),delt_h,Cdiss_f(i-1,1),Cdiss_p(i-1,1),Cdiss_cell(i-1,1));
                %                 [Cdiss(i,1),Cdiss_f(i,1),Cdiss_p(i,1),Cdiss_cell(i,1),Ctotal(i,1)] = ...
                %                     solve_Cdiss_model2(diss_parameters,uptake_parms,V,...
                %                     areat(i,1),delt_h,Cdiss_f(i-1,1),Cdiss_p(i-1,1),...
                %                     Cdiss_cell(i-1,1),Ctotal(i-1,1),part_conc);
                coeffd = 2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(i,1),Cdiss_p(i,1),V);
                
                dens_values = num_dens_txdp(i).values;
                switch agg_yn
                    
                    case 'agg_n'
                        for j=1:Ndp
                            %  for k=1:Nx
                            
                            derdp_values(:,j) = dens_values(:,j)*coeffd/(rho_dp(j,1)*10^(-15));
                            % end
                            
                        end
                    case 'agg_y'
                        for j=1:Ndp
                            %  for k=1:Nx
                            if(dp(j,1)>dp_prim)
                                derdp_values(:,j) = dens_values(:,j)*coeffd*(dp(j,1)/dp_prim)/(rhop_prim*10^(-15));
                            else
                                derdp_values(:,j) = dens_values(:,j)*coeffd/(rho_dp(j,1)*10^(-15));
                            end
                            % end
                            
                        end
                        
                end
                
                derdp_values(:,1) = 0;
                derdp_num_dens_txdp(i).values = derdp_values;
                clear derdp_values;
                clear dens_values;
                
                derdp_num_dens_txdp(i).values(:,:) = derdp(derdp_num_dens_txdp(i).values(:,:),del_dp,Ndp);
                
                
                
                % set the derivative to zero at the lowest value, to not allow
                % particles of the lowest size to dissolve.
                
                % derdp_num_dens_txdp(i).values(:,1) = 0;
                for j=1:Ndp
                    % Implicit 2nd order Adam-Bashforth (Crank-Nicolson)
                    % u(n+1) = u(n) +
                    % delta_t*[f(t(n+1),u(n+1))+f(t(n),u(n))]/2
                    num_dens_txdp(i).values(:,j) = num_dens_txdp(i-1).values(:,j)+ delt_h*(derdp_num_dens_txdp(i).values(:,j)+derdp_num_dens_txdp(i-1).values(:,j))/2;
                    
                end
                
                
            end
            
            
            
            % Evaluate N_{i+1;q} using equation , with N_{i+1/2;q} as the
            % initial solution
            
            %             Cdissq = Cdiss(i,1);
            %             Cdisscellq = Cdiss_cell(i,1);
            %
            
            num_dens_txdp_half(:,:) = num_dens_txdp(i).values(:,:);  % #/nm^2
            
            %             delt_sec = delt_h * 3600;
            
            t0 = time.values_hr(i-1,1)*3600; % lower time limit (sec)
            tf = time.values_hr(i,1)*3600;  % upper time limit (sec)
            
            %t0 = time.values_hr(i-1,1); % lower time limit (h)
            %   tf = time.values_hr(i,1);  % upper time limit (h)
            
            tdata1 = linspace(t0,tf,4); % time array(sec)
            
            ind_end = length(tdata1);
            tdata = tdata1;
            % x in m units
            delx = x(2)-x(1);  % m
            % Parallelizing the for loop for improving performance
            
            sol_values = zeros(Nx,Ndp);
            
            xtstart = tic;
            
            solinit = zeros(Nx,Ndp);
            
            for ind1=1:Nx
                for ind2 = 1:Ndp
                    vpi = pi*dp(ind2,1)*dp(ind2,1)*dp(ind2,1)/6; % volume of particle with diameter dp_i in nm^3
                    rhopi = rho_dp(ind2,1)*(1e-21);        % density in g/nm^3
                    % SA units : m^2
                    % integrate_density_dp(temp,dp) unit: g/nm
                    % 1g/(nm m^2) = 1e6 ug/(1e-9 m^3) = 1e6 ug/(1e-9 * 1e6 cm^3) = 1e9 ug/ml
                    solinit(ind1,ind2) =  num_dens_txdp_half(ind1,ind2)*vpi*rhopi*(dp(2)-dp(1))*(1e9)/SA; % mass per unit physical volume (ug/ml)
                end
            end
            parfor j=1:Ndp
                %       for j = 1:Ndp
                
                diff = diff_dp(j,1) ;  % m^2/s
                vsed = vsed_dp(j,1);    % m/s
                xprime = x';
                xdata = x';
                
                %init_sol = solinit(:,j); % #ug/ml
                init_sol = solinit(:,j)/C0; % dimensionless concentration
                xprime = xprime/L;
                xdata = xdata/L;
                delx = xdata(2)-xdata(1);
                tdata1 = tdata/(L/vsed);  % time scale = sedimentation time scale
                
                diff = diff/(L*vsed);
                vsed = 1;
                
                sol = pdepe(0,@partpdex,@particx,@partbcx,xprime,tdata1,[],diff,vsed,delx,bctype,init_sol,xdata);
                %sol = pdepe(0,@partpdex,@particx,@partbcx,xprime,tdata,[],diff,vsed,delx,bctype,init_sol,xdata);
                %  sol_values(:,j)=sol(ind_end,:)';
                sol_values(:,j)=C0*sol(ind_end,:)'; % ug/ml
                
            end
            
            for ind1=1:Nx
                for ind2 = 1:Ndp
                    vpi = pi*dp(ind2,1)*dp(ind2,1)*dp(ind2,1)/6; % volume of particle with diameter dp_i in nm^3
                    rhopi = rho_dp(ind2,1)*(1e-21);        % density in g/nm^3
                    % SA units : m^2
                    % integrate_density_dp(temp,dp) unit: g/nm
                    % 1g/(nm m^2) = 1e6 ug/(1e-9 m^3) = 1e6 ug/(1e-9 * 1e6 cm^3) = 1e9 ug/ml
                    
                    num_dens_txdp(i).values(ind1,ind2) =  sol_values(ind1,ind2)/(vpi*rhopi*(dp(2)-dp(1))*(1e9)/SA); % number density (#/nm^2)
                end
            end
            
            
            %                         parfor j=1:Ndp
            %
            %                             diff = diff_dp(j,1) ;  % m^2/s
            %                             vsed = vsed_dp(j,1);    % m/s
            %                             %        diff = diff_dp(j,1)*3600*(1e18) ;  % nm^2/h
            %                             % vsed = vsed_dp(j,1)*3600*(1e9);    % nm/h
            %                             xprime = x';
            %                             xdata = x';
            %                             %xprime = x_nm';
            %                             %xdata = x_nm';
            %                             % convert units from #/nm^2 to #/m^2
            %                             init_sol = 1e18*num_dens_txdp_half(:,j); % #/m^2
            %                             % init_sol = num_dens_txdp_half(:,j); % #/nm^2
            %
            %                             sol = pdepe(0,@partpdex,@particx,@partbcx,xprime,tdata,[],diff,vsed,delx,bctype,init_sol,xdata);
            %                             sol_values(:,j) = (1e-18)*sol(ind_end,:)';   % #/nm^2
            %                             % num_dens_txdp(i).values(:,j) = (1e-18)*sol(ind_end,:)';   % #/nm^2
            %                             % num_dens_txdp(i).values(:,j) = sol(ind_end,:)';   % #/nm^2
            %                             %               disp(['jth iteration # = ',num2str(j), ' completed']);
            %                         end
            %
            %
            %                         num_dens_txdp(i).values = sol_values;
            xtelapsed = toc(xtstart);
            disp(['Time taken to solve in x space = ',num2str(xtelapsed)]);
            
            
            
            partNumq = integrate_density_xdp(num_dens_txdp(i).values,dp,x_nm);
            %[areatbyN0,partSAq] = surface_area(Nx,Ndp,dp,num_dens_txdp(i).values(:,:),x_nm,N0,dp_prim);
            
            disp(['number of particles in liquid media = ',num2str(partNumq)]);
            %              dispout = ['number of particles in liquid media = ',num2str(partNumq)];
            %                 fprintf(logfid,'%s\n',dispout);
            
            [areat_N0_ratio(i,1),areat(i,1)] = surface_area(agg_yn,rho_dp,Nx,Ndp,dp,num_dens_txdp(i).values(:,:),x_nm,N0,rhop_prim,dp_prim);
            partSAq = areat(i,1);
            
            
            
            
            % Calculate total concentration, which should match the initial
            % concentration.
            % Total Conc = C0 = Cdiss_f + Cdiss_p +Cpart_liquid + Cpart_cells
            % + Cion_cells
            
            % Calculate concentration of particles in the liquid media
            %  - First, calculate the total number of particles
            % - Second, calculate the mass and then concentration.
            
            
            output_conc = calcPartConc(num_dens_txdp,num_dens_dp_cell,i,dp,x_nm,SA,rho_dp,...
                V,Cdiss_f,Cdiss_p,Cdiss_cell,diss_parameters,delt_h,agg_yn,rhop_prim,dp_prim);
            
            num_dens_dp_cell(i,1).values = output_conc.num_dens_dp_cell_it;
            total_conc = output_conc.t_totalConc;
            Cdiss_ftmp = Cdiss_f(i,1);
            Cdiss_f(i,1) = Cdiss_f(i,1)+C0-total_conc;   % correct for the error in total concentration
            if(Cdiss_f(i,1)<0)
                Cdiss_f(i,1)=Cdiss_ftmp;
                Cdiss_ptmp = Cdiss_p(i,1);
                Cdiss_p(i,1)=Cdiss_p(i,1)+C0-total_conc;
                if(Cdiss_p(i,1)<0)
                    Cdiss_p(i,1)=Cdiss_ptmp;
                end
            end
            Cdiss(i,1) = Cdiss_f(i,1)+Cdiss_p(i,1);
            %             part_conc = output_conc.t_partConcC +output_conc.t_partConcL;
            total_conc = output_conc.t_partConcC +output_conc.t_partConcL + Cdiss(i,1)+Cdiss_cell(i,1);
            Ctotal(i,1) = total_conc;
            disp(['Cdiss = ', num2str(Cdiss(i,1)), ' ug/ml']);
            disp(['Cdiss_f = ', num2str(Cdiss_f(i,1)), ' ug/ml']);
            disp(['Cdiss_p = ', num2str(Cdiss_p(i,1)), ' ug/ml']);
            disp(['Cdiss_cell = ', num2str(Cdiss_cell(i,1)), ' ug/ml']);
            disp(['Cpart_C = ', num2str(output_conc.t_partConcC), ' ug/ml']);
            disp(['Cpart_L = ', num2str(output_conc.t_partConcL), ' ug/ml']);
            
            disp(['Ctotal = ', num2str(Ctotal(i,1)), ' ug/ml']);
            
            if(q>1)
                %                 n = 0;
                %                 for j=1:Nx
                %                     for k=1:Ndp
                %                         n = n + 1;
                %                         residual(n) = abs(num_dens_txdp_q1(j,k)-num_dens_txdp(i).values(j,k));
                %                     end
                %                 end
                
                % error = max(residual);
                % error1 = max(residual);
                % error2 = abs(Cdissq-Cdissqminus1);
                % error3 = abs(Cdisscellq-Cdisscellqminus1);
                % error4 = abs(partNumq  - partNumqminus1);
                error5 = abs(partSAq  - partSAqminus1);
                
                
                %error12345 =[error1;error2;error3;error4;error5];
                
                %error = max(error12345);
                error=max(error5);
                errort(i,1) = error;
                
                %end
                error_conc = abs(total_conc-C0);
                disp(['q iteration # = ', num2str(q), ' error = ', num2str(error)]);
                disp(['q iteration # = ', num2str(q), ' error in total conc = ', num2str(error_conc)]);
                disp(['q iteration # = ', num2str(q), 'error conc tol = ', num2str(error_conc_tol)]);
                disp(['difference in error for total concentration between iterations = ', num2str(abs(error_conc-error_conc_prev))]);
                
                if(abs(error_conc-error_conc_prev)<1e-7)
                    error_conc_tol = error_conc+1e-10;
                end
                error_conc_prev = error_conc;
            end
        end
    end
    
    
    
end
matlabpool close;
%delete(gcp('nocreate'));
%----------------------TIME INTEGRATION ENDS HERE--------------------------


time_elapsed = toc(time_start)/3600;

disp(['total CPU time (in hours): ' num2str(time_elapsed)]);

%  dispout = ['total CPU time (in hours): ' num2str(time_elapsed)];
%                 fprintf(logfid,'%s\n',dispout);

save(results_matfile,'N0','errort','C0','time_elapsed',...
    'Cdiss_f','Cdiss_p','Cdiss_cell',...
    'parameter','diss_parameters','uptake_parms','dp_prim','diff_dp','vsed_dp',...
    'x','x_nm','dp','rho_dp','time','num_dens_init',...
    'areat_N0_ratio','n0_dp','num_dens_txdp','Cdiss','areat','tdata','-v7.3');

secondary_outputs = calculateSecondaryOutputs(results_matfile);
save(results_matfile,'secondary_outputs','-append');

%Write out the results into a an excel spreadsheet
disp('Writing out the results.');
% dispout = ['Writing the parameter and results files.'];
%                 fprintf(logfid,'%s\n',dispout);
% fclose(logfid);
% fname = results_matfile;
% writeParameters(parameter,diss_parameters,uptake_parms,fname);

use_xlwrite = parameter.use_xlwrite;
writeParticleIonSolutionVsTime(secondary_outputs,use_xlwrite,fname);


end

%-----------------FUNCTION calcPartConc-----------------------------------
%   - Function to calculate the mass and concentration of particles in the
%   liquid and cellular media.

function output = calcPartConc(num_dens_txdp,num_dens_dp_cell,it,dp,x_nm,SA,rho_dp,...
    V,Cdiss_f,Cdiss_p,Cdiss_cell,diss_parameters,delt_h,agg_yn,rhop_prim,dp_prim)
% INPUT:
% - num_dens_txdp: number density of particles in the liquid media
Ndp = length(dp);
Nx= length(x_nm);
del_dp = (dp(2)-dp(1)); %nm


%  num_dens_dp_cell = zeros(Ndp,1);

% Calculate concentration of particle in liquid media as a function of
% x
output.tx_partConcL = zeros(1,Nx);
temp = zeros(Ndp,1);

for j=1:Nx
    for i = 1:Ndp
        vpi = pi*dp(i,1)*dp(i,1)*dp(i,1)/6; % volume of particle with diameter dp_i in nm^3
        rhopi = rho_dp(i,1)*(1e-21);        % density in g/nm^3
        
        temp(i,1) = num_dens_txdp(it).values(j,i)*vpi*rhopi; % mass density (g/nm^2) of particles having diameter dp_
        %      temp = num_dens_txdp(it).values(j,:).*vpi.*rhopi; % CHECK IF THIS IS CORRECT
        
    end
    % SA units : m^2
    % integrate_density_dp(temp,dp) unit: g/nm
    % 1g/(nm m^2) = 1e6 ug/(1e-9 m^3) = 1e6 ug/(1e-9 * 1e6 cm^3) = 1e9 ug/ml
    
    output.tx_partConcL(1,j) = integrate_density_dp(temp,dp)*(1e9)/SA;  % mass per unit physical volume (ug/ml)
end
clear temp;
% Calculate total mass and concentration of particles in the liquid
% media
delx_cm = (x_nm(2)-x_nm(1))*1e-7;  % cm
SA_cm2 = SA*1e4; % cm^2

% total mass of particles in the liquid media
output.t_partMassL =(output.tx_partConcL(1,1)*0.5+ sum(output.tx_partConcL(1,2:Nx-1)) +...
    output.tx_partConcL(1,Nx)*0.5)*SA_cm2*delx_cm;

% total concentration of particles (ug/ml) in the liquid media, as a
% function of time.
output.t_partConcL =  output.t_partMassL/V;

% calculate total mass and concentration of particles deposited on cells

delv = num_dens_txdp(it,1).values-num_dens_txdp(it-1).values;
delv_dp = integrate_density_x(delv,x_nm);

DdpDt_termi = zeros(Ndp,1);
DdpDt_termiminus1 = zeros(Ndp,1);
% how many particles of size dp are lost due to dissolution in delta t.
switch agg_yn
    case 'agg_n'
        for j = 1:Ndp
            rhopi = rho_dp(j,1)*(1e-15);        % Density of particles with diameter dp_i, in ug/nm^3
            %if(j>1)
            DdpDt_termiminus1(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(it-1,1),Cdiss_p(it-1,1),V)/rhopi; % nm/hr
            DdpDt_termi(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(it,1),Cdiss_p(it,1),V)/rhopi; % nm/hr
            %else
            % DdpDt_termiminus1(j,1) = 0;
            % DdpDt_termi(j,1) = 0;
            % end
        end
        
    case 'agg_y'
        for j = 1:Ndp
            
            %if(j>1)
            if(dp(j,1)>dp_prim)
                rhopi = rhop_prim*(1e-15);
                DdpDt_termiminus1(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(it-1,1),Cdiss_p(it-1,1),V)*(dp(j,1)/dp_prim)/rhopi; % nm/hr
                DdpDt_termi(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(it,1),Cdiss_p(it,1),V)*(dp(j,1)/dp_prim)/rhopi; % nm/hr
            else
                rhopi = rho_dp(j,1)*(1e-15);        % Density of particles with diameter dp_i, in ug/nm^3
                % density of particles of size less than the
                % primary particle size is same as the primary
                % particle density.
                
                DdpDt_termiminus1(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(it-1,1),Cdiss_p(it-1,1),V)/rhopi; % nm/hr
                DdpDt_termi(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(it,1),Cdiss_p(it,1),V)/rhopi; % nm/hr
            end
            %else
            % DdpDt_termiminus1(j,1) = 0;
            % DdpDt_termi(j,1) = 0;
            % end
            
        end
        
end
func_dpiminus1 = zeros(Nx,Ndp);

func_dpi = zeros(Nx,Ndp);
% delfunc_dpi = zeros(Nx,Ndp);
% delfunc_dpiminus1 = zeros(Nx,Ndp);
for j=1:Nx
    
    for k=1:Ndp
        if(k>1)
            
            func_dpiminus1(j,k) = (num_dens_txdp(it-1,1).values(j,k))*DdpDt_termiminus1(k,1);
            func_dpi(j,k) = (num_dens_txdp(it,1).values(j,k))*DdpDt_termi(k,1);
        else
            func_dpiminus1(j,k) = 0;
            func_dpi(j,k) = 0;
            
        end
    end
    
    
end

delfunc_dpi = -derdp(func_dpi,del_dp,Ndp);
delfunc_dpiminus1 = -derdp(func_dpiminus1,del_dp,Ndp);
%delfunc_dpi(:,1) = 0;
%delfunc_dpiminus1(:,1) = 0;
deldp_dpx = 0.5*(delfunc_dpiminus1+delfunc_dpi)*delt_h;
deldp_dp = integrate_density_x(deldp_dpx,x_nm);
delvalue = delv_dp-deldp_dp;
% ia = delvalue<0;
num_dens_dp_cell_it = num_dens_dp_cell(it-1,1).values-delvalue;
%output.td_partNumDensC = num_dens_dp_cell;
mass_temp = zeros(Ndp,1);

for i = 1:Ndp
    vpi = pi*dp(i,1)*dp(i,1)*dp(i,1)/6; % volume of particle with diameter dp_i in nm^3
    rhopi = rho_dp(i,1)*(1e-21);        % density in g/nm^3
    %surfai = pi*dp(i,1)*dp(i,1);  % surface area of particles, in nm^2.
    mass_temp(i,1) = num_dens_dp_cell_it(i,1)*vpi*rhopi; % mass density (g/nm) of particles having diameter dp_
end

output.t_partMassC = sum(mass_temp)*(1e6)*(dp(2)-dp(1)); % ug
output.t_partConcC = output.t_partMassC/V; % ug/ml

% total concentration
output.t_totalConc = Cdiss_f(it,1) + Cdiss_p(it,1)+ Cdiss_cell(it,1)+...
    output.t_partConcC + output.t_partConcL;
output.num_dens_dp_cell_it = num_dens_dp_cell_it;
end
%-----------------FUNCTION calculateSecondaryOutputs----------------------
function [output] = calculateSecondaryOutputs(matfname)
% Output variables calculated

% output.td_partNumDensL: particle number density in liquid media vs.
%                       time and diameter
% output.t_partNumL:  number of particles in liquid media vs. time
% output.tx_partConcL: mass concentration of particles in liquid media vs.
%                       time and x
% output.t_partMassL: mass of particles in liquid media vs. time
% output.t_partConcL: mass concentration of particles in liquid media vs.
%                       time
% output.t_partSAL:  surface area of particles in liquid media vs. time
%
% output.td_partNumDensC: particle number density deposited on cells vs.
%                       time and diameter
% output.t_partNumC: number of particles deposited on cells vs. time
% output.t_partMassC: mass of particles deposited on cells vs. time
% output.t_partConcC: mass concentration of particles deposited on cells  vs.
%                       time
% output.t_partSAC:  surface area of particles deposited on cells vs. time
%
% output.t_partNumTot: total number of particles in the system vs. time
%
% output.t_ionConcFreeL: mass concentration of ions in free state in liquid
%                       media vs. time
% output.t_ionConcBndL:  mass concentration of ions in protein-bound state in liquid
%                       media vs. time
% output.t_ionConcTotL: total concentration of ions in liquid
%                       media vs. time
% output.t_ionConcC:     mass concentration of ions deposited in cells vs. time
%
% output.t_ionMassFreeL: mass of ions in free state in liquid media vs.
%                           time
% output.t_ionMassBndL: mass of ions in protein-bound state in liquid media vs. time
% output.t_ionMassTotL: total mass of ions in liquid media vs. time
% output.t_ionMassC:        total mass of ions in cells vs. time

% load the mat file containing the results of the simulation.
load([matfname '.mat']);
SA = parameter.media_surfacearea; % % Surface area of the medium perpendicular to
% x axis direction (m^2)
% C0 = parameter.conc;
% L = parameter.media_height;
V = parameter.media_volume;
Nt = length(time.values_hr);
Ndp = length(dp);
Nx = length(x_nm);
% del_x = (x_nm(2,1)-x_nm(1,1));  % nm
% delt_s = (time.values_hr(2,1)-time.values_hr(1,1))*3600;
del_dp = (dp(2)-dp(1)); %nm


agg_yn = parameter.agg_yn;
dp_prim = parameter.ptcl_diameter; % nm
rhop_prim = parameter.ptcl_density; % g/cc
%
% visc = parameter.media_viscosity;
% T = parameter.media_temperature;

% kf=diss_parameters.kf;
% kp = diss_parameters.kp;
% n = diss_parameters.n;
% n2 = diss_parameters.n2;
% kp2 = diss_parameters.kp2;
% P0 = diss_parameters.P0;
% Cdiss_f_sat = diss_parameters.Cfsat;
delt_h = time.delt_h;

num_dens_dp_cell(Nt,1) = struct;
num_dens_dp_liquidmedia(Nt,1) = struct;
for i=1:Nt
    num_dens_dp_cell(i,1).values = zeros(Ndp,1);
    num_dens_dp_liquidmedia(i,1).values = zeros(Ndp,1);
    
end
temp = zeros(Ndp,1);
for i = 1:Ndp
    
    vpi = pi*dp(i,1)*dp(i,1)*dp(i,1)/6; % Volume of particle with diameter dp_i, in nm^3
    rhopi = rho_dp(i,1)*(1e-21);        % Density of particles with diameter dp_i, in g/nm^3
    temp(i,1) = n0_dp(i,1)*vpi*rhopi; % Mass of particles with diameter dp_i, in g
end

% denom_N0 = integrate_density_dp(temp,dp);   % Area under the temp vs Dp curve

% N0 = C0*V*(1e-6)/denom_N0;  % Initial number of particles

clear temp;
clear vpi;
clear rhopi;

% Calculate the total number of particles at time t = 0.
%num_part = integrate_density_xdp(num_dens_txdp(1,1).values,dp,x_nm);

% Get the number density in the liquid media, in dp space
for i=1:Nt
    num_dens_dp_liquidmedia(i,1).values = integrate_density_x(num_dens_txdp(i,1).values,x_nm);
end
% total count of particles in liquid media, and deposited on cells.
output.t_partNumL = zeros(Nt,1);
output.t_partNumC = zeros(Nt,1);
output.t_partNumTot = zeros(Nt,1);

i = 1;
output.t_partNumL(i,1) = integrate_density_dp(num_dens_dp_liquidmedia(i,1).values,dp);
output.t_partNumC(i,1) = integrate_density_dp(num_dens_dp_cell(i,1).values,dp);
output.t_partNumTot(i,1) =  output.t_partNumL(i,1) +   output.t_partNumC(i,1);

for i=2:Nt
    
    delv = num_dens_txdp(i,1).values-num_dens_txdp(i-1).values;
    delv_dp = integrate_density_x(delv,x_nm);
    
    DdpDt_termi = zeros(Ndp,1);
    DdpDt_termiminus1 = zeros(Ndp,1);
    % how many particles of size dp are lost due to dissolution in delta t.
    switch agg_yn
        case 'agg_n'
            for j = 1:Ndp
                
                rhopi = rho_dp(j,1)*(1e-15);        % Density of particles with diameter dp_i, in ug/nm^3
                %if(j>1)
                DdpDt_termiminus1(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(i-1,1),Cdiss_p(i-1,1),V)/rhopi; % nm/hr
                DdpDt_termi(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(i,1),Cdiss_p(i,1),V)/rhopi; % nm/hr
                %else
                % DdpDt_termiminus1(j,1) = 0;
                % DdpDt_termi(j,1) = 0;
                % end
            end
        case 'agg_y'
            for j = 1:Ndp
                if (dp(j,1)>dp_prim)
                    rhopi = rhop_prim*(1e-15);        % Density of primary particle, in ug/nm^3
                    %if(j>1)
                    DdpDt_termiminus1(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(i-1,1),Cdiss_p(i-1,1),V)*(dp(j,1)/dp_prim)/rhopi; % nm/hr
                    DdpDt_termi(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(i,1),Cdiss_p(i,1),V)*(dp(j,1)/dp_prim)/rhopi; % nm/hr
                else
                    rhopi = rho_dp(j,1)*(1e-15);        % Density of particles with diameter dp_i, in ug/nm^3
                    %if(j>1)
                    DdpDt_termiminus1(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(i-1,1),Cdiss_p(i-1,1),V)/rhopi; % nm/hr
                    DdpDt_termi(j,1) = -2*DdpDt_dissterm_model1(diss_parameters,Cdiss_f(i,1),Cdiss_p(i,1),V)/rhopi; % nm/hr
                    %else
                    % DdpDt_termiminus1(j,1) = 0;
                    % DdpDt_termi(j,1) = 0;
                end
            end
            
    end
    func_dpiminus1 = zeros(Nx,Ndp);
    
    func_dpi = zeros(Nx,Ndp);
    %     delfunc_dpi = zeros(Nx,Ndp);
    %     delfunc_dpiminus1 = zeros(Nx,Ndp);
    for j=1:Nx
        for k=1:Ndp
            if(k>1)
                func_dpiminus1(j,k) = (num_dens_txdp(i-1,1).values(j,k))*DdpDt_termiminus1(k,1);
                func_dpi(j,k) = (num_dens_txdp(i,1).values(j,k))*DdpDt_termi(k,1);
            else
                func_dpiminus1(j,k) = 0;
                func_dpi(j,k) = 0;
            end
        end
        
    end
    delfunc_dpi = -derdp(func_dpi,del_dp,Ndp);
    delfunc_dpiminus1 = -derdp(func_dpiminus1,del_dp,Ndp);
    % delfunc_dpi(:,1) = 0;
    %delfunc_dpiminus1(:,1) = 0;
    deldp_dpx = 0.5*(delfunc_dpiminus1+delfunc_dpi)*delt_h;
    deldp_dp = integrate_density_x(deldp_dpx,x_nm);
    % delvalue = delv_dp-deldp_dp;
    delvalue = delv_dp-deldp_dp;
    %     ia = delvalue<0;
    num_dens_dp_cell(i,1).values = num_dens_dp_cell(i-1,1).values-delvalue;
    %output.t_partNumC(i,1) = output.t_partNumC(i-1,1) - integrate_density_xdp(delv,dp,x_nm);
    
    % Calculate the total number of particles in the liquid, deposited on
    % the cells, and combined
    output.t_partNumL(i,1) = integrate_density_dp(num_dens_dp_liquidmedia(i,1).values,dp);
    output.t_partNumC(i,1) = integrate_density_dp(num_dens_dp_cell(i,1).values,dp);
    output.t_partNumTot(i,1) =  output.t_partNumL(i,1) +   output.t_partNumC(i,1);
end

% % total count of particles in liquid media, and deposited on cells.
% output.t_partNumL = zeros(Nt,1);
% output.t_partNumC = zeros(Nt,1);
% output.t_partNumTot = zeros(Nt,1);

% for i=1:Nt
%     %output.t_partNumL(i,1) = integrate_density_xdp(num_dens_txdp(i,1).values,dp,x_nm);
%     output.t_partNumL(i,1) = integrate_density_dp(num_dens_dp_liquidmedia(i,1).values,dp);
%     output.t_partNumC(i,1) = integrate_density_dp(num_dens_dp_cell(i,1).values,dp);
%     output.t_partNumTot(i,1) =  output.t_partNumL(i,1) +   output.t_partNumC(i,1);
% end

% Store number density of particles in dp space, deposited on cells
output.td_partNumDensC = num_dens_dp_cell;
clear num_dens_dp_cell;
% Store number density of particles in dp space, in liquid media
output.td_partNumDensL = num_dens_dp_liquidmedia;
clear num_dens_dp_liquidmedia;

%  Calculate particle mass concentration in liquid media as a function of t and x.
output.tx_partConcL = zeros(Nt,Nx);
for k = 1:Nt
    
    for j=1:Nx
        for i = 1:Ndp
            vpi = pi*dp(i,1)*dp(i,1)*dp(i,1)/6; % volume of particle with diameter dp_i in nm^3
            rhopi = rho_dp(i,1)*(1e-21);        % density in g/nm^3
            
            temp(i,1) = num_dens_txdp(k).values(j,i)*vpi*rhopi; % mass density (g/nm^2) of particles having diameter dp_
        end
        % SA units : m^2
        % integrate_density_dp(temp,dp) unit: g/nm
        % 1g/(nm m^2) = 1e6 ug/(1e-9 m^3) = 1e6 ug/(1e-9 * 1e6 cm^3) = 1e9 ug/ml
        output.tx_partConcL(k,j) = integrate_density_dp(temp,dp)*(1e9)/SA;  % mass per unit physical volume (ug/ml)
    end
end


% total mass of particles (ug) in the liquid media,as a function of time.
output.t_partMassL = zeros(Nt,1);
delx_cm = (x_nm(2)-x_nm(1))*1e-7;  % cm
SA_cm2 = SA*1e4; % cm^2
for i=1:Nt
    output.t_partMassL(i,1) = (output.tx_partConcL(i,1)*0.5 + sum(output.tx_partConcL(i,2:Nx-1))+...
        output.tx_partConcL(i,Nx)*0.5)*SA_cm2*delx_cm;
end

% total concentration of particles (ug/ml) in the liquid media, as a
% function of time.
output.t_partConcL =  output.t_partMassL/V;

% total surface area of particles in the liquid media, as a function of
% time
output.t_partSAL = zeros(Nt,1);
for k=1:Nt
    switch agg_yn
        case 'agg_n'
            for i=1:Ndp
                
                surfai = pi*dp(i,1)*dp(i,1);  % surface area of particles, in nm^2.
                temp(i,1) = output.td_partNumDensL(k).values(i,1)*surfai;
            end
        case 'agg_y'
            for i=1:Ndp
                if (dp(i,1)>dp_prim)
                    surfai = ((dp(i,1)/dp_prim)^3)*(rho_dp(i,1)/rhop_prim)*pi*dp_prim*dp_prim;
                    
                    %surfai = pi*dp(i,1)*dp(i,1);  % surface area of particles, in nm^2.
                    
                else
                    surfai = pi*dp(i,1)*dp(i,1);  % surface area of particles, in nm^2.
                    
                end
                temp(i,1) = output.td_partNumDensL(k).values(i,1)*surfai;
            end
            
    end
    output.t_partSAL(k,1) = (sum(temp)*(dp(2)-dp(1)))*(1e-14);  % cm^2
    clear temp;
end
% mass (ug), concentration (ug/ml), and surface area (cm^2) of particles
% deposited on cells as a function of time.
output.t_partMassC = zeros(Nt,1);
output.t_partConcC = zeros(Nt,1);
output.t_partSAC = zeros(Nt,1);
for k=1:Nt
    mass_temp = zeros(Ndp,1);
    surf_temp = zeros(Ndp,1);
    switch agg_yn
        case 'agg_n'
            for i = 1:Ndp
                vpi = pi*dp(i,1)*dp(i,1)*dp(i,1)/6; % volume of particle with diameter dp_i in nm^3
                rhopi = rho_dp(i,1)*(1e-21);        % density in g/nm^3
                surfai = pi*dp(i,1)*dp(i,1);  % surface area of particles, in nm^2.
                mass_temp(i,1) = output.td_partNumDensC(k).values(i,1)*vpi*rhopi; % mass density (g/nm) of particles having diameter dp_
                surf_temp(i,1) = output.td_partNumDensC(k).values(i,1)*surfai; % surface area (nm^2/nm) of particles having diameter dp_
            end
        case 'agg_y'
            for i = 1:Ndp
                
                vpi = pi*dp(i,1)*dp(i,1)*dp(i,1)/6; % volume of particle with diameter dp_i in nm^3
                rhopi = rho_dp(i,1)*(1e-21);        % density in g/nm^3
                if (dp(i,1)>dp_prim)
                    surfai = ((dp(i,1)/dp_prim)^3)*(rho_dp(i,1)/rhop_prim)*pi*dp_prim*dp_prim;
                else
                    surfai = pi*dp(i,1)*dp(i,1);  % surface area of particles, in nm^2.
                end
                mass_temp(i,1) = output.td_partNumDensC(k).values(i,1)*vpi*rhopi; % mass density (g/nm) of particles having diameter dp_
                surf_temp(i,1) = output.td_partNumDensC(k).values(i,1)*surfai; % surface area (nm^2/nm) of particles having diameter dp_
                
                % vpi = pi*dp(i,1)*dp(i,1)*dp(i,1)/6; % volume of particle with diameter dp_i in nm^3
                % rhopi = rho_dp(i,1)*(1e-21);        % density in g/nm^3
                % surfai = pi*dp(i,1)*dp(i,1);  % surface area of particles, in nm^2.
                % mass_temp(i,1) = output.td_partNumDensC(k).values(i,1)*vpi*rhopi; % mass density (g/nm) of particles having diameter dp_
                % surf_temp(i,1) = output.td_partNumDensC(k).values(i,1)*surfai; % surface area (nm^2/nm) of particles having diameter dp_
                
            end
    end
    
    output.t_partMassC(k,1) = sum(mass_temp)*(1e6)*(dp(2)-dp(1)); % ug
    output.t_partConcC(k,1) = output.t_partMassC(k,1)/V; % ug/ml
    output.t_partSAC(k,1) = (sum(surf_temp)*(dp(2)-dp(1)))*(1e-14); % cm^2
    clear mass_temp;
    clear surf_temp;
end


%Calculate ion mass concentration (ug/ml) in liquid media and cells as a function of time
output.t_ionConcFreeL = Cdiss_f;
output.t_ionConcBndL = Cdiss_p;
output.t_ionConcTotL=Cdiss;
output.t_ionConcC = Cdiss_cell;

% ion mass (units: ug) in liquid media and in cells as a function of time
output.t_ionMassFreeL = Cdiss_f*V;  % V in ml units
output.t_ionMassBndL = Cdiss_p*V;
output.t_ionMassTotL = Cdiss*V;
output.t_ionMassC = Cdiss_cell*V;


output.t_totalConc = output.t_partConcL+output.t_partConcC + output.t_ionConcTotL + output.t_ionConcC;

output.time_hr = time.values_hr;
output.time_units ='hr';
output.dp_units = 'nm';
output.x_units = 'nm';
output.mass_concentration_units = 'ug/ml';
output.mass_units = 'ug';
output.dp_nm = dp;
output.x_nm = x_nm;

output.errort = errort;
end


% ------------------------FUNCTION surface_area--------------------------
%   Calculates the the total surface of particles
%   and the total surface area of particles divided by initial number of
%   particles
function [areat_N0_ratio,areat] = surface_area(agg_yn,rho_dp,Nx,Ndp,dp,num_dens,x_nm,N0,rhop_prim,dp_prim)

% Calculate total surface area (nm^2) of particles divided by initial number of
% particles (areat_N0), and total surface area (areat).
% Surface area of agglomerates is calculated in terms of the primary particle diameter (dp_prim, in nm)
% 	To define the agglomerate surface area in terms of the primary particle
%surface area. First, the number of primary particles (say N)
%that form a mass equivalent to the mass of one agglomerate is calculated,
%as shown below:
%
% N = (Dagg/D)^3 * (rho_agg/rho), where
%
% Dagg = the diameter of the agglomerate
% D = the diameter of the primary particle
% rho_agg = density of the agglomerate
% rho = density of the primary particle
%
% Then the surface area of the agglomerate can be calculated as,
%
% SA_agg = N*pi*D^2
% = (Dagg/D)^3 * (rho_agg/rho) * pi * D^2.



temp(:,:) = num_dens;
switch agg_yn
    case 'agg_n'
        % Treat secondary particles with size greater than the primary undissolved
        % particle size as non-porous particles (not agglomerates).
        % Particles with size less than primary particle size are always
        % treated as non-porous particles.
        for i=1:Nx
            for j=1:Ndp
                
                temp(i,j) = temp(i,j)*pi*dp(j,1)*dp(j,1);   % nm^2
            end
            
        end
        
    case 'agg_y'
        %Treat secondary particles with size greater than primary undissolved
        %particle size as agglomerates.
        
        % Particles with size less than primary particle size are always
        % treated as non-porous particles.
        
        for i=1:Nx
            for j=1:Ndp
                if(dp(j,1)<=dp_prim)
                    temp(i,j) = temp(i,j)*pi*dp(j,1)*dp(j,1);   % nm^2
                else
                    temp(i,j) = temp(i,j)*((dp(j,1)/dp_prim)^3)*(rho_dp(j,1)/rhop_prim)*pi*dp_prim*dp_prim;   % nm^2
                end
                
            end
        end
end
% total surface area of particles (nm^2)
areat = integrate_density_xdp(temp,dp,x_nm);

% total surface area (nm^2) per particle
areat_N0_ratio = areat/N0;   % nm^2

end
% -------------------------FUNCTION partpdex-------------------------------
%   Defines the terms of the partial differential equation (PDE).
function [c,f,s] = partpdex(x,t,u,DuDx,diff,vsed,delx,bctype,init_sol,xdata)
c = 1;
f = diff*DuDx-vsed*u;
s = 0;

end
% ------------------------FUNCTION particx---------------------------------
%   Sets the initial solution for the solving the PDE.
function u0 = particx(x,diff,vsed,delx,bctype,init_sol,xdata)
%i0 = round(x/delx+1);
i0 = xdata==x;
% initial solution at a position along x-axis
u0 = init_sol(i0,1);
%  disp(['i0= ',num2str(i0),', x(m) = ',num2str(x)]);
%  disp('Next call');
end
% ------------------------FUNCTION partbcx---------------------------------
%   Sets the boundary conditions for solving the PDE.
function [pl,ql,pr,qr] = partbcx(xl,ul,xr,ur,t,diff,vsed,delx,bctype,init_sol,xdata)
pl = 0;
ql = 1;
pr = bctype*ur;  % if sticky, bctype = 1; if no flux, bctype = 0
qr = -bctype+1;

end
% -------------------------FUNCTION derdp----------------------------------
%   Calculates the derivative of a variable in Dp space.
function derdp_txdp  = derdp(num_dens_xdp,del_dp,Ndp)

% del_dp, nm
dnum_dens_derdp_txdp = num_dens_xdp;

dnum_dens_derdp_txdp(:,1) = (num_dens_xdp(:,2)-num_dens_xdp(:,1))/del_dp;
%dnum_dens_derdp_txdp(:,Ndp) = (num_dens_xdp(:,Ndp)-num_dens_xdp(:,Ndp-1))/del_dp;
dnum_dens_derdp_txdp(:,Ndp) = (0-num_dens_xdp(:,Ndp))/del_dp;
for j=2:Ndp-1
    
    % Simplest Upstream (Upwind) differencing scheme (1st order)
    % dq/dt + a dq/dx = 0
    % q(i,n+1) = q(i,n) -(deltat*a(n)/delx)*(q(i,n)-q(i-1,n))  if a(n) > 0
    %q(i,n+1) = q(i,n) -(deltat*a(n)/delx)*(q(i+1,n)-q(i,n))  if a(n) < 0
    % The second scheme is used here
    dnum_dens_derdp_txdp(:,j) = (num_dens_xdp(:,j+1)-num_dens_xdp(:,j))/(del_dp);
end
derdp_txdp = dnum_dens_derdp_txdp;

end
%-------------------FUNCTION integrate_density_x---------------------------
%   Integrates 2D density in x
function area_x = integrate_density_x(dens,x)
Nx = length(dens(:,1));
Ndp = length(dens(1,:));

% dens - #/nm^2
% x - nm
% dp - nm


area_x = zeros(Ndp,1);
for i=1:Ndp
    Y = dens(:,i);
    % area_x(i,1) = sum(Y(:,1))*(x(2)-x(1));
    area_x(i,1) = (Y(1,1)*0.5+sum(Y(2:Nx-1,1))+Y(Nx,1)*0.5)*(x(2)-x(1));
end

end
% ------------------FUNCTION integrate_density_dp--------------------------
%   Integrates 1D density in Dp space.
function area_dp = integrate_density_dp(dens,dp)

%area_dp = trapz(dp,dens);
Ndp = length(dp);
del_dp = dp(2)-dp(1);
area_dp = sum(dens(:,1))*(dp(2)-dp(1));

%area_dp = dens(1,1)*del_dp*0.5+sum(dens(2:Ndp-1,1))*del_dp+dens(Ndp,1)*del_dp*0.5;
end
% ------------------FUNCTION integrate_density_xdp-------------------------
%   Integrates 2D density in x and Dp space.
function area_xdp = integrate_density_xdp(dens,dp,x)
Nx = length(dens(:,1));
Ndp = length(dens(1,:));

% dens - #/nm^2
% x - nm
% dp - nm

area_x = zeros(Ndp,1);
for i=1:Ndp
    Y = dens(:,i);
    area_x(i,1) = (Y(1,1)*0.5+sum(Y(2:Nx-1,1))+Y(Nx,1)*0.5)*(x(2)-x(1));
end

area_xdp = sum(area_x)*(dp(2)-dp(1));  % assuming uniform spacing for the x and dp values.

end
% -----------------------FUNCTION diffusivity------------------------------
%   Calculates the diffusivity of particles/agglomerates of diameter dp nm
function diff_dp = diffusivity(T,visc,dp,pth)

% diffusivity = R T/(3 NA pi mu diamp)
% R - gas constant (L kPa/K/mol)
% T  - temperature (K)
% diamp - particle diameter (nm)
% mu (visc) - viscosity (N s/m2)
% NA - Avogadro's Number
% visc -  viscosity (N s/m2)
% constants (use SI units)
% pth - protein thickness (nm)

%dpm = dp * 1e-9;              % particle diameter (m)
dpm = (dp+2*pth)*1e-9; % diameter of particles coated with a protein layer of thickness, pth.

rpm = dpm/2;                     % particle radius (m)
R=8.314472;                         % Gas Constant (L kPa/K/mol)
NA=6.022e23;                         % Avogadro's Number
% g=9.8;                      % acceleration due to gravity (m/s2)

diff_dp=R*T/(NA*6*pi*visc*rpm);             % diffusivity (m2/s)

end
% --------------------FUNCTION sedimentation_velocity----------------------
%   Calculates the sedimentational velocity of particles / agglomerates
%   of diameter dp nm.
function [vsed_dp,rho_dp]= sedimentation_velocity(agg_yn,DF,PF,dw,visc,dp,dp_prim,rhop_prim,pth,rhop_protein)
% constants (use SI units)
% DF - Fractal Dimension (only relevant to agglomerates)
% PF - Packing Factor (only relevant to agglomerates)
% dw -Density of media (g/mL)
% visc -  viscosity (N s/m2)
% dp - diameter of dissolving secondary particles (aggregates or non-porous particles)
% dp_prim - diameter of the undissolved primary particle (nm), constant
% rhop_prim - density of the undissolved primary particle (g/cc)
% pth - thickness of the protein layer coating the surface of particles
% (nm)

% particles with size less than primary particle size are always
% treated as non-porous particles


%dpm = dp * 1e-9;              % particle diameter (m)
dpm = (dp + 2*pth)*1e-9;
rpm = dpm/2;                     % particle radius (m)
% R=8.314472;                         % Gas Constant (L kPa/K/mol)
% N=6.022e23;                         % Avogadro's Number
g=9.8;                      % Gravity (m/s2)

switch agg_yn
    
    case 'agg_n'
        
        % Treat secondary particles with size greater than the primary undissolved
        % particle size as non-porous particles (not agglomerates).
        % Particles with size less than primary particle size are always
        % treated as non-porous particles.
        if (pth==0)
            vsed_dp =2.*g*(rhop_prim-dw)*rpm*rpm/9./visc*1e3;   % sedimentation (m/s)
            rho_dp = rhop_prim;
        else
            
            vsed_dp =2.*g*(rhop_protein-dw)*rpm*rpm/9./visc*1e3;   % sedimentation (m/s)
            rho_dp = rhop_prim;
            
        end
        
    case 'agg_y'
        %Treat secondary particles with size greater than primary undissolved
        %particle size as agglomerates.
        
        % Particles with size less than primary particle size are always
        % treated as non-porous particles.
        
        if (dp <= dp_prim)
            
            vsed_dp=2.*g*(rhop_prim-dw)*rpm*rpm/9./visc*1e3;   % sedimentation (m/s)
            rho_dp = rhop_prim;
        elseif  (dp > dp_prim)
            
            %             aggnp = int32(PF*(dp/dp_prim)^DF);
            aggdiam = dp* 1e-9;          % agg diameter in m
            dp_prim_m = dp_prim* 1e-9;  % primary particle diameter in m
            %             aggrad = aggdiam/2;                    % agg radius(m)
            aggpor = 1.-(aggdiam/dp_prim_m)^(DF-3.);   % agg porosity
            aggdens = rhop_prim * (1.-aggpor)+dw*aggpor;   % agg density
            
            vsed_dp=g*(aggdens-dw)*(aggdiam)^(DF-1.)*(dp_prim_m)^(3.-DF)/18./visc*1e3; %m/s
            
            rho_dp = aggdens;
        end
        
end

end
%------------------------FUNCTION setMesh----------------------------------
%   Sets up the mesh in x-Dp space
function [output] = setMesh(dp_u,dp_l,dp_avg,del_dp,Nx,L)

% Think of every point dp(i)in discretized dp space refers to the mid point of a segment of size delta dp.
% The lower and upper limits of dp should be chosen such that there are no
% particles of sizes greater than dp(n)+delta_dp/2 and sizes smaller than
% dp(1)-delta_dp/2.  The mesh points will go from dp(1) to dp(n).

% consider dp_u +del_dp/2 as maximum, and dp_l-del_dp/2 as minimum.
% Approximate dp(1) = dp_l, and dp(n) = dp_u.
%Ndp = round((dp_u-dp_l)/del_dp + 1);   % number of grid points along the dp coordinate axis
%dp = zeros(Ndp,1);              % variable to hold the discretized values of particle diameter (nm) in the range [dp_l,dp_u]
% for i=1:Ndp
%     dp(i,1) = dp_l + (i-1)*del_dp;  % nm
% end
dp_i = dp_l:del_dp:dp_u; % variable to hold the discretized values of particle diameter (nm) in the range [dp_l,dp_u]
dp = dp_i';
Ndp= length(dp); % number of grid points along the dp coordinate axis

% Nondimensionalize the diameter (Dp) dimension variables, dp and del_dp,
% by dp_avg.
%   - Variables ending with '_dim' means they are dimensionless variables

dp_dim = dp/dp_avg;         % dimensionless diameter
del_dp_dim = del_dp/dp_avg; % dimensionless grid spacing along Dp coordinate axis


x_max_dim = 1;              % Dimensionless value of x at x = L
% (top surface of the medium). L is used to
% non-dimensionalize the x values.
delx_dim = x_max_dim/(Nx-1);% Dimensionless uniform spacing between the grid points along x

%x: Variable to hold the discretized values of particle position (m) in the range [0,L]
x_dim = zeros(Nx,1);
% Calculate the dimensionless values for the particle position in the x direction
for i=1:Nx
    x_dim(i,1) = (i-1)*delx_dim;
end
%x_dim(end,1) = x_max_dim;

x = x_dim*L;        % x in m units
x_nm = x*1e9;       % x in nm units

output.Ndp = Ndp;
output.Nx = Nx;
output.dp = dp;
output.delx_dim = delx_dim;
output.x = x;
output.x_nm=x_nm;
output.dp_dim = dp_dim;
output.del_dp_dim = del_dp_dim;
output.x_dim = x_dim;

end
% --------------FUNCTION initializeNumFracDensDpspace----------------------
% Initializes the number fraction density function in Dp space
function [output] = initializeNumFracDensDpspace(size_distribution_type,Ndp,dp,dp_avg,dp_sd,dp_init_l,dp_init_u)

n0_dp = zeros(Ndp,1);           % variable to hold the initial particle number fraction density as a function of particle diameter only

% Set up the values of initial number fraction density (probability density) as a Gaussian
% distribution in Dp space (nm)

% first find the probability at every point dp (this point refers to the
% mid point of a segment of size delta dp.
% The lower and upper limits of dp should be chosen such that there are no
% particles of sizes greater than dp(n)+delta_dp/2 and sizes smaller than
% dp(1)-delta_dp/2.  The mesh points will go from dp(1) to dp(n).

% Assume Gaussian distribution for the particle number fraction density,
% calculate particle number fraction with dp-delp/2 and dp+delp/2
% (probability of finding a particle within dp +/- delp/2.
prob = zeros(Ndp,1);
avg_dp = zeros(Ndp,1);
del_dp = dp(2,1) - dp(1,1);

switch size_distribution_type
    
    case 'normal'
        f = @(x) exp(-((x-dp_avg).^2)/(2*dp_sd*dp_sd))/(dp_sd*sqrt(2*pi));
        s = @(x) exp(-((x-dp_avg).^2)/(2*dp_sd*dp_sd)).*x./(dp_sd*sqrt(2*pi));
    case 'log-normal'
        f = @(x) exp(-((log(x)-dp_avg).^2)/(2*dp_sd*dp_sd))./(x*dp_sd*sqrt(2*pi));
        s = @(x) exp(-((log(x)-dp_avg).^2)/(2*dp_sd*dp_sd))./(dp_sd*sqrt(2*pi));
        
end

for i=1:Ndp
    if(dp(i)>=dp_init_l && dp(i)<= dp_init_u)
        prob(i,1) = integral(f,dp(i,1)-0.5*del_dp,dp(i,1)+0.5*del_dp);
        if(prob(i,1)~=0)
            avg_dp(i,1) = integral(s,dp(i,1)-0.5*del_dp,dp(i,1)+0.5*del_dp)/prob(i,1);
        else
            avg_dp(i,1) = dp(i,1);
        end
        n0_dp(i,1) = prob(i,1)/del_dp;
    end
end


% Calculate the area of the initial number fraction density function in
% Dp space.

area_n0_dp = integrate_density_dp(n0_dp,dp);    % units - 1/nm

% Normalize n0_dp by area_n0_dp so that the total area is unity.
%   - Normalization is essential if dp_sd < del_dp in order for the
%     integral of the number fraction density to be 1.

n0_dp = n0_dp/area_n0_dp;

% Calculate and check the area of the normalized ,initial number fraction density
% (should equal to 1).

area_n0_dp_norm = integrate_density_dp(n0_dp,dp);

% Nondimensionalize the diameter (Dp) dimension variables, dp and del_dp,
% by dp_avg.
%   - Variables ending with '_dim' means they are dimensionless variables

output.n0_dp = n0_dp;
output.area_n0_dp = area_n0_dp;
output.area_n0_dp_norm=area_n0_dp_norm;
output.avg_dp = avg_dp;
end

%-----------------------FUNCTION initializeNumDensXDPSpace-----------------
%   Calcualtes the initital number density of the particles in the liquid
%   medium.
function [num_dens_dp0,num_dens_init,area_num_dens] = initializeNumDensXDPSpace(N0,dp,x_nm,n0_dp,Nx,Ndp,delx_dim,L)

num_dens_dp0 = N0*n0_dp;

% Initialize the number fraction density in the x-Dp space to zero



num_dens_init.data = zeros(Nx,Ndp);     % units in /nm^2
% Set up the initial solution of the particle number density function (dimensionless) in x-Dp
% space, using n0_dp
%   -  % Particles of a given diameter, dp, are uniformly distributed in x

% The number density is considered uniform in the external
% coordinate dimension (x).

for j=1:Ndp
    %     for  i=1:Nx-1
    %         num_dens_init.data(i,j) = num_dens_dp0(j,1)/(delx_dim*L*1e9*(Nx-1));
    %
    %     end
    num_dens_init.data(1,j) = 0.5*num_dens_dp0(j,1)/(0.5*delx_dim*L*1e9*(Nx-1));
    tmp = 0*0.5*num_dens_dp0(j,1)/(delx_dim*L*1e9*(Nx-1)*(Nx-2));
    for  i=2:Nx-1
        num_dens_init.data(i,j) = num_dens_dp0(j,1)/(delx_dim*L*1e9*(Nx-1)) +tmp;
        
    end
    num_dens_init.data(Nx,j) = 0.5*num_dens_dp0(j,1)/(0.5*delx_dim*L*1e9*(Nx-1));
end

% Initialize the solution of num_frac_dens in #/nm^2 units

num_dens = num_dens_init.data;

% Integrate the number fraction density in x-Dp space.
% The value should be equal to N0; if not, divide the number density by
% the value.
area_num_dens = integrate_density_xdp(num_dens,dp,x_nm);
num_dens_init.data = N0*num_dens/area_num_dens;
end
