% File: solve_Cdiss_model1.m
% Author: Dennis G. Thomas
% Created date: December 2013
% Last modified: July 29, 2017

function  [Cdiss_total,Cdiss_f,Cdiss_p,Cdiss_cell] = solve_Cdiss_model1(diss_parms,uptake_parms,V,Areat,delt_h,Cdiss_f_in,Cdiss_p_in,Cdiss_cell_in)
% Cdiss_f, dissolved concentration of unbound ions, in ug/ml
%  Cdiss_p, dissolved concentration of protein-bound ions, in ug/ml
%  Cdiss_total, total dissolved ion concentration, in ug/ml
% Cdiss_cell, dissolved concentration of unbound ions in cell, in ug/ml
% V, media volume in ml
% Areat, total surface area of particles in the medium (nm^2) at time t
% delt_h, time step in hr
% uptake_parms: parameters of the cell uptake model for ions

% parameters of the empirical dissolution model for ions.
P0 = diss_parms.P0; % percentage of FBS in solution
kf = diss_parms.kf; % rate constant for dissolution of particles to free ions (in nm hr^-1) [diffusivity,D/distance,h]
Cfsat = diss_parms.Cfsat; % saturated concentration of free ions in the media (in ug/ml)
kp = diss_parms.kp; % rate constant for slow transfer of ions from the particle surface to the proteins (in nm hr-1);
kp2 = diss_parms.kp2; % rate constant for the initial fast transfer of ions from the particle surface to the proteins (in nm hr-1)
kfp = diss_parms.kfp;
kp2f = diss_parms.kp2f;
n = diss_parms.n; % concentration of binding sites available on the proteins per FBS for the slow binding period of the ions% (in ug ml^-1 FBS%^-1)
n2 = diss_parms.n2; % concentration of binding sites available on the proteins per FBS for the fast binding period of the ions% (in ug ml^-1 FBS%^-1)

% parameters of the cell uptake model for ions
sa2 = uptake_parms.sa2; % cm^2          % 2 - cells, 1 - media
dis2 = uptake_parms.dis2; % cm
d12 = uptake_parms.d12; % cm^2/hr
pc21 = uptake_parms.pc21; % no units
cellv = uptake_parms.cellv; % mL

tspan = [0:delt_h:delt_h]';

    function dCdt = ode(t,C)
        
        dCdt = zeros(3,1);
        
        % C(1) : unbound ion concentration in the media
        % C(2) : protein bound ion concentration in the media
        % C(3) : ion concentration in the cells 
    
    % Note: The concentration is defined as mass per media volume
        
   Areat_by_V = Areat/V;  % nm^2/ml
        
   dCdt(1) = 0;
      dCdt(2) = 0;
      dCdt(3) = 0;
   if (C(1) <= Cfsat)
       dCdt(1) = dCdt(1) + kf*Areat_by_V*(Cfsat-C(1))+ kp2f*C(2).*(Cfsat-C(1));
      %dCdt(1) = dCdt(1) + kf*Areat*(Cfsat-C(1))+ kp2f*C(2).*(Cfsat-C(1));
       
        dCdt(2) = dCdt(2) -kp2f*C(2).*(Cfsat-C(1));
   end
   
   if(C(2) <= n*P0)
      dCdt(1) = dCdt(1) -kfp*C(1).*(n*P0-C(2));
       dCdt(2) = dCdt(2) + kfp*C(1).*(n*P0-C(2))+ kp*Areat_by_V*(n*P0-C(2));
        %dCdt(2) = dCdt(2) + kfp*C(1).*(n*P0-C(2))+ kp*Areat*(n*P0-C(2));
   end
   
   if(C(2) <= n2*P0)
     dCdt(2) = dCdt(2) + kp2*Areat_by_V*(n2*P0-C(2)); 
    %  dCdt(2) = dCdt(2) + kp2*Areat*(n2*P0-C(2)); 
   end
    % ion uptake term    
    dCdt(1) = dCdt(1)-(1/V)*d12*sa2*(C(1)-C(3)*(V/cellv)/pc21)/dis2;
    dCdt(2) = dCdt(2)-(1/V)*d12*sa2*(C(2)-C(3)*(V/cellv)/pc21)/dis2;
     dCdt(3) = dCdt(3)+(1/V)*d12*sa2*(C(1)+C(2)-C(3)*(V/cellv)/pc21)/dis2;
    
    end

[tout,Cout] =  ode45(@ode,tspan,[Cdiss_f_in Cdiss_p_in Cdiss_cell_in]);

Cdiss_f = Cout(end,1);
Cdiss_p = Cout(end,2);
Cdiss_cell = Cout(end,3);
Cdiss_total = Cdiss_f + Cdiss_p;

end
